@@ -50,16 +50,40 @@ class ProxyError(Error):
 MIME_BOUNDARY = 'END_OF_PART'
 
 
+def get_headers(http_response):
+  """Retrieves all HTTP headers from an HTTP response from the server.
+  
+  This method is provided for backwards compatibility for Python2.2 and 2.3.
+  The httplib.HTTPResponse object in 2.2 and 2.3 does not have a getheaders
+  method so this function will use getheaders if available, but if not it
+  will retrieve a few using getheader.
+  """
+  if hasattr(http_response, 'getheaders'):
+    return http_response.getheaders()
+  else:
+    headers = []
+    for header in (
+        'location', 'content-type', 'content-length', 'age', 'allow',
+        'cache-control', 'content-location', 'content-encoding', 'date',
+        'etag', 'expires', 'last-modified', 'pragma', 'server',
+        'set-cookie', 'transfer-encoding', 'vary', 'via', 'warning',
+        'www-authenticate', 'gdata-version'):
+      value = http_response.getheader(header, None)
+      if value is not None:
+        headers.append((header, value))
+    return headers
+
+
 class HttpRequest(object):
   """Contains all of the parameters for an HTTP 1.1 request.
- 
+
   The HTTP headers are represented by a dictionary, and it is the
   responsibility of the user to ensure that duplicate field names are combined
   into one header value according to the rules in section 4.2 of RFC 2616.
   """
   method = None
   uri = None
- 
+
   def __init__(self, uri=None, method=None, headers=None):
     """Construct an HTTP request.
 
@@ -80,7 +104,7 @@ class HttpRequest(object):
 
   def add_body_part(self, data, mime_type, size=None):
     """Adds data to the HTTP request body.
-   
+
     If more than one part is added, this is assumed to be a mime-multipart
     request. This method is designed to create MIME 1.0 requests as specified
     in RFC 1341.
@@ -153,7 +177,7 @@ class HttpRequest(object):
   def add_form_inputs(self, form_data,
                       mime_type='application/x-www-form-urlencoded'):
     """Form-encodes and adds data to the request body.
-    
+
     Args:
       form_data: dict or sequnce or two member tuples which contains the
                  form keys and values.
@@ -174,6 +198,26 @@ class HttpRequest(object):
     new_request._body_parts = self._body_parts[:]
     return new_request
 
+  def _dump(self):
+    """Converts to a printable string for debugging purposes.
+    
+    In order to preserve the request, it does not read from file-like objects
+    in the body.
+    """
+    output =  'HTTP Request\n  method: %s\n  url: %s\n  headers:\n' % (
+        self.method, str(self.uri))
+    for header, value in self.headers.iteritems():
+      output += '    %s: %s\n' % (header, value)
+    output += '  body sections:\n'
+    i = 0
+    for part in self._body_parts:
+      if isinstance(part, (str, unicode)):
+        output += '    %s: %s\n' % (i, part)
+      else:
+        output += '    %s: <file like object>\n' % i
+      i += 1
+    return output
+
 
 def _apply_defaults(http_request):
   if http_request.uri.scheme is None:
@@ -189,7 +233,7 @@ class Uri(object):
   host = None
   port = None
   path = None
- 
+
   def __init__(self, scheme=None, host=None, port=None, path=None, query=None):
     """Constructor for a URI.
 
@@ -213,7 +257,7 @@ class Uri(object):
       self.port = port
     if path:
       self.path = path
-     
+
   def _get_query_string(self):
     param_pairs = []
     for key, value in self.query.iteritems():
@@ -232,7 +276,7 @@ class Uri(object):
       return '?'.join([path, param_string])
     else:
       return path
-     
+
   def _to_string(self):
     if self.scheme is None and self.port == 443:
       scheme = 'https'
@@ -252,7 +296,7 @@ class Uri(object):
 
   def __str__(self):
     return self._to_string()
-     
+
   def modify_request(self, http_request=None):
     """Sets HTTP request components based on the URI."""
     if http_request is None:
@@ -277,7 +321,7 @@ class Uri(object):
 
   def parse_uri(uri_string):
     """Creates a Uri object which corresponds to the URI string.
- 
+
     This method can accept partial URIs, but it will leave missing
     members of the Uri unset.
     """
@@ -319,7 +363,7 @@ class HttpResponse(object):
   status = None
   reason = None
   _body = None
- 
+
   def __init__(self, status=None, reason=None, headers=None, body=None):
     self._headers = headers or {}
     if status is not None:
@@ -331,7 +375,7 @@ class HttpResponse(object):
         self._body = body
       else:
         self._body = StringIO.StringIO(body)
-         
+
   def getheader(self, name, default=None):
     if name in self._headers:
       return self._headers[name]
@@ -340,7 +384,7 @@ class HttpResponse(object):
 
   def getheaders(self):
     return self._headers
-   
+
   def read(self, amt=None):
     if self._body is None:
       return None
@@ -350,19 +394,36 @@ class HttpResponse(object):
       return self._body.read(amt)
 
 
+def _dump_response(http_response):
+  """Converts to a string for printing debug messages.
+  
+  Does not read the body since that may consume the content.
+  """
+  output = 'HttpResponse\n  status: %s\n  reason: %s\n  headers:' % (
+      http_response.status, http_response.reason)
+  headers = get_headers(http_response)
+  if isinstance(headers, dict):
+    for header, value in headers.iteritems():
+      output += '    %s: %s\n' % (header, value)
+  else:
+    for pair in headers:
+      output += '    %s: %s\n' % (pair[0], pair[1])
+  return output
+
+
 class HttpClient(object):
   """Performs HTTP requests using httplib."""
   debug = None
- 
+
   def request(self, http_request):
-    return self._http_request(http_request.method, http_request.uri, 
+    return self._http_request(http_request.method, http_request.uri,
                               http_request.headers, http_request._body_parts)
 
   Request = request
 
   def _get_connection(self, uri, headers=None):
     """Opens a socket connection to the server to set up an HTTP request.
-    
+
     Args:
       uri: The full URL for the request as a Uri object.
       headers: A dict of string pairs containing the HTTP headers for the
@@ -383,11 +444,11 @@ class HttpClient(object):
 
   def _http_request(self, method, uri, headers=None, body_parts=None):
     """Makes an HTTP request using httplib.
-   
+
     Args:
       method: str example: 'GET', 'POST', 'PUT', 'DELETE', etc.
       uri: str or atom.http_core.Uri
-      headers: dict of strings mapping to strings which will be sent as HTTP 
+      headers: dict of strings mapping to strings which will be sent as HTTP
                headers in the request.
       body_parts: list of strings, objects with a read method, or objects
                   which can be converted to strings using str. Each of these
@@ -395,8 +456,9 @@ class HttpClient(object):
     """
     if isinstance(uri, (str, unicode)):
       uri = Uri.parse_uri(uri)
+
     connection = self._get_connection(uri, headers=headers)
- 
+
     if self.debug:
       connection.debuglevel = 1
 
@@ -430,7 +492,7 @@ class HttpClient(object):
     if body_parts:
       for part in body_parts:
         _send_data_part(part, connection)
-   
+
     # Return the HTTP Response from the server.
     return connection.getresponse()
 
