@@ -26,6 +26,7 @@ __author__ = 'j.s@google.com (Jeff Scudder)'
 
 
 import gdata.client
+import gdata.gauth
 import gdata.spreadsheets.data
 import atom.data
 import atom.http_core
@@ -39,19 +40,19 @@ WORKSHEET_URL = ('http://spreadsheets.google.com/feeds/worksheets/'
                  '%s/private/full/%s')
 TABLES_URL = 'http://spreadsheets.google.com/feeds/%s/tables'
 RECORDS_URL = 'http://spreadsheets.google.com/feeds/%s/records/%s'
+RECORD_URL = 'http://spreadsheets.google.com/feeds/%s/records/%s/%s'
 
 
 class SpreadsheetsClient(gdata.client.GDClient):
   api_version = '3'
-  auth_serice = 'wise'
-  auth_scopes = ['https://spreadsheets.google.com/feeds/',
-                 'http://spreadsheets.google.com/feeds/']
+  auth_service = 'wise'
+  auth_scopes = gdata.gauth.AUTH_SCOPES['wise']
 
   def get_spreadsheets(self, auth_token=None,
                        desired_class=gdata.spreadsheets.data.SpreadsheetsFeed,
                        **kwargs):
     """Obtains a feed with the spreadsheets belonging to the current user.
-    
+
     Args:
       auth_token: An object which sets the Authorization HTTP header in its
                   modify_request method. Recommended classes include
@@ -77,7 +78,7 @@ class SpreadsheetsClient(gdata.client.GDClient):
                      desired_class=gdata.spreadsheets.data.WorksheetsFeed,
                      **kwargs):
     """Finds the worksheets within a given spreadsheet.
-   
+
     Args:
       spreadsheet_key: str, The unique ID of this containing spreadsheet. This
                        can be the ID from the URL or as provided in a
@@ -106,7 +107,7 @@ class SpreadsheetsClient(gdata.client.GDClient):
   def add_worksheet(self, spreadsheet_key, title, rows, cols,
                     auth_token=None, **kwargs):
     """Creates a new worksheet entry in the spreadsheet.
-    
+
     Args:
       spreadsheet_key: str, The unique ID of this containing spreadsheet. This
                        can be the ID from the URL or as provided in a
@@ -239,7 +240,7 @@ class SpreadsheetsClient(gdata.client.GDClient):
   def add_record(self, spreadsheet_key, table_id, fields,
                  title=None, auth_token=None, **kwargs):
     """Adds a new row to the table.
-    
+
     Args:
       spreadsheet_key: str, The unique ID of this containing spreadsheet. This
                        can be the ID from the URL or as provided in a
@@ -300,6 +301,41 @@ class SpreadsheetsClient(gdata.client.GDClient):
 
   GetRecords = get_records
 
+  def get_record(self, spreadsheet_key, table_id, record_id,
+                 desired_class=gdata.spreadsheets.data.Record,
+                 auth_token=None, **kwargs):
+    """Retrieves a single record from the table.
+
+    Args:
+      spreadsheet_key: str, The unique ID of this containing spreadsheet. This
+                       can be the ID from the URL or as provided in a
+                       Spreadsheet entry.
+      table_id: str, The ID of the table within the worksheet whose records
+                we would like to fetch. The table ID can be found using the
+                get_table_id method of a gdata.spreadsheets.data.Table.
+      record_id: str, The ID of the record within this table which we want to
+                 fetch. You can find the record ID using get_record_id() on
+                 an instance of the gdata.spreadsheets.data.Record class.
+      desired_class: class descended from atom.core.XmlElement to which a
+                     successful response should be converted. If there is no
+                     converter function specified (converter=None) then the
+                     desired_class will be used in calling the
+                     atom.core.parse function. If neither
+                     the desired_class nor the converter is specified, an
+                     HTTP reponse object will be returned. Defaults to
+                     gdata.spreadsheets.data.RecordsFeed.
+      auth_token: An object which sets the Authorization HTTP header in its
+                  modify_request method. Recommended classes include
+                  gdata.gauth.ClientLoginToken and gdata.gauth.AuthSubToken
+                  among others. Represents the current user. Defaults to None
+                  and if None, this method will look for a value in the
+                  auth_token member of SpreadsheetsClient."""
+    return self.get_entry(RECORD_URL % (spreadsheet_key, table_id, record_id),
+                          desired_class=desired_class, auth_token=auth_token,
+                          **kwargs)
+
+  GetRecord = get_record
+
 
 class SpreadsheetQuery(gdata.client.Query):
 
@@ -323,7 +359,7 @@ class SpreadsheetQuery(gdata.client.Query):
     gdata.client._add_query_param('title-exact', self.title_exact,
                                   http_request)
     gdata.client.Query.modify_request(self, http_request)
-  
+
   ModifyRequest = modify_request
 
 
@@ -340,14 +376,14 @@ class ListQuery(gdata.client.Query):
       order_by: str Specifies what column to use in ordering the entries in
                 the feed. By position (the default): 'position' returns
                 rows in the order in which they appear in the GUI. Row 1, then
-                row 2, then row 3, and so on. By column: 
+                row 2, then row 3, and so on. By column:
                 'column:columnName' sorts rows in ascending order based on the
                 values in the column with the given columnName, where
                 columnName is the value in the header row for that column.
       reverse: str Specifies whether to sort in descending or ascending order.
                Reverses default sort order: 'true' results in a descending
-               sort; 'false' (the default) results in an ascending sort. 
-      sq: str Structured query on the full text in the worksheet. 
+               sort; 'false' (the default) results in an ascending sort.
+      sq: str Structured query on the full text in the worksheet.
           [columnName][binaryOperator][value]
           Supported binaryOperators are:
           - (), for overriding order of operations
@@ -366,7 +402,7 @@ class ListQuery(gdata.client.Query):
     gdata.client._add_query_param('reverse', self.reverse, http_request)
     gdata.client._add_query_param('sq', self.sq, http_request)
     gdata.client.Query.modify_request(self, http_request)
-  
+
   ModifyRequest = modify_request
 
 
@@ -411,5 +447,5 @@ class CellQuery(gdata.client.Query):
     gdata.client._add_query_param('return-empty', self.return_empty,
                                   http_request)
     gdata.client.Query.modify_request(self, http_request)
-  
+
   ModifyRequest = modify_request
